// Environment-dependent compilation.
// It's possible to bind variables from 'here' (i.e. this runtime environment) inside a compiled function. The way we do it is to create a closure using a gensym. (Another reason that gensyms
// must really be unique.) Here's the idea. We use the Function constructor to create an outer function, bind a bunch of variables directly within that scope, and return the function we're
// compiling. The variables correspond to gensyms placed in the code, so the code will have closure over those variables.

// An optional second parameter 'environment' can contain a hash of variable->value bindings. These will be defined as locals within the compiled function.

// New in caterwaul 0.6.5 is the ability to specify a 'this' binding to set the context of the expression being evaluated.

  caterwaul_global.compile = function (tree, environment) {
    var vars = [], values = [], bindings = merge({}, this._environment || {}, environment || {}, tree.bindings()), s = gensym();
    for (var k in bindings) if (own.call(bindings, k)) vars.push(k), values.push(bindings[k]);

    var variable_definitions = map(function (v) {return v === 'this' ? '' : 'var ' + v + '=' + s + '.' + v}, vars).join(';'),
        return_statement     = 'return(' + tree.toString() + ')',
        function_body        = variable_definitions + ';' + return_statement;

    try       {return (new Function(s, function_body)).call(bindings['this'], bindings)}
    catch (e) {throw new Error(e + ' while compiling ' + function_body)}};
// Generated by SDoc 
